@@ -1,9 +1,18 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include <sirius.hpp>
+#include <testing.hpp>
 
 using namespace sirius;
 
-void
-test1()
+int
+test_phase()
 {
     int N = 30000000;
     std::vector<r3::vector<double>> a(N);
@@ -33,12 +42,11 @@ test1()
     }
     printf("(default schedule) speed: %f million phase-factors / sec.\n", N * 10 / t1 / 1000000);
     printf("(static schedule) speed: %f million phase-factors / sec.\n", N * 10 / t2 / 1000000);
+    return 0;
 }
 
 int
 main(int argn, char** argv)
 {
-    cmd_args args(argn, argv, {{}});
-
-    test1();
+    return call_test("test_phase", test_phase);
 }
