@@ -199,7 +199,8 @@ PYBIND11_MODULE(py_sirius, m)
             .def("processing_unit", py::overload_cast<std::string>(&Simulation_context::processing_unit))
             .def("gvec", &Simulation_context::gvec, py::return_value_policy::reference_internal)
             .def("full_potential", &Simulation_context::full_potential)
-            .def_property_readonly("hubbard_correction", &Simulation_context::hubbard_correction)
+            .def("hubbard_correction", py::overload_cast<>(&Simulation_context::hubbard_correction, py::const_))
+            .def("hubbard_correction", py::overload_cast<bool>(&Simulation_context::hubbard_correction))
             .def_property_readonly("kT", py::overload_cast<>(&Simulation_context::smearing_width, py::const_))
             .def_property_readonly("smearing_type", py::overload_cast<>(&Simulation_context::smearing, py::const_))
             .def("unit_cell", py::overload_cast<>(&Simulation_context::unit_cell, py::const_),
@@ -211,8 +212,10 @@ PYBIND11_MODULE(py_sirius, m)
             .def("aw_cutoff", py::overload_cast<>(&Simulation_context::aw_cutoff, py::const_))
             .def("aw_cutoff", py::overload_cast<double>(&Simulation_context::aw_cutoff))
             .def("num_spinors", &Simulation_context::num_spinors)
-            .def("num_mag_dims", &Simulation_context::num_mag_dims)
+            .def("num_mag_dims", py::overload_cast<>(&Simulation_context::num_mag_dims, py::const_))
+            .def("num_mag_dims", py::overload_cast<int>(&Simulation_context::num_mag_dims))
             .def_property_readonly("gamma_point", py::overload_cast<>(&Simulation_context::gamma_point, py::const_))
+            .def("gamma_point", py::overload_cast<bool>(&Simulation_context::gamma_point))
             .def("update", &Simulation_context::update)
             .def("use_symmetry", py::overload_cast<>(&Simulation_context::use_symmetry, py::const_))
             .def("processing_unit_memory_t", &Simulation_context::processing_unit_memory_t)
