@@ -1,5 +1,5 @@
 from .py_sirius import InverseS_k, S_k, Precond_us, Hamiltonian0
-from .coefficient_array import threaded
+# from .coefficient_array import threaded
 from scipy.sparse.linalg import LinearOperator
 import numpy as np
 
@@ -29,7 +29,7 @@ class S_operator(KpointOperatorBase):
 
     def apply(self, cn):
         """"""
-        out = type(cn)(dtype=cn.dtype, ctype=cn.ctype)
+        out = type(cn)()
         for key in cn.keys():
             out[key] = np.array(self.ops[key].apply(np.asfortranarray(cn[key])))
         return out
@@ -38,7 +38,7 @@ class S_operator(KpointOperatorBase):
         def _matvec(X):
             return np.array(self.ops[key].apply(np.asfortranarray(X)))
         n = self.ops[key].size
-        return LinearOperator(dtype=np.complex, shape=(n, n), matvec=_matvec, rmatvec=_matvec)
+        return LinearOperator(dtype=np.complex128, shape=(n, n), matvec=_matvec, rmatvec=_matvec)
 
     def __matmul__(self, cn):
         return self.apply(cn)
@@ -57,7 +57,7 @@ class Sinv_operator(KpointOperatorBase):
                 )
 
     def apply(self, cn):
-        out = type(cn)(dtype=cn.dtype, ctype=cn.ctype)
+        out = type(cn)()
         for key in cn.keys():
             out[key] = np.array(self.ops[key].apply(np.asfortranarray(cn[key])))
         return out
@@ -66,7 +66,7 @@ class Sinv_operator(KpointOperatorBase):
         def _matvec(X):
             return np.array(self.ops[key].apply(np.asfortranarray(X)))
         n = self.ops[key].size
-        return LinearOperator(dtype=np.complex, shape=(n, n), matvec=_matvec, rmatvec=_matvec)
+        return LinearOperator(dtype=np.complex128, shape=(n, n), matvec=_matvec, rmatvec=_matvec)
 
     def __matmul__(self, cn):
         return self.apply(cn)
@@ -91,7 +91,7 @@ class US_Precond(KpointOperatorBase):
                 )
 
     def apply(self, cn):
-        out = type(cn)(dtype=cn.dtype, ctype=cn.ctype)
+        out = type(cn)()
         for key in cn.keys():
             out[key] = np.array(self.ops[key].apply(np.asfortranarray(cn[key])))
         return out
@@ -100,7 +100,7 @@ class US_Precond(KpointOperatorBase):
         def _matvec(X):
             return np.array(self.ops[key].apply(np.asfortranarray(X)))
         n = self.ops[key].size
-        return LinearOperator(dtype=np.complex, shape=(n, n), matvec=_matvec, rmatvec=_matvec)
+        return LinearOperator(dtype=np.complex128, shape=(n, n), matvec=_matvec, rmatvec=_matvec)
 
     def __matmul__(self, cn):
         return self.apply(cn)
